<?php

namespace Drupal\Tests\crouton\Functional;

use Drupal\node\Entity\Node;
use Drupal\menu_link_content\Entity\MenuLinkContent;

use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\system\Functional\Menu\AssertBreadcrumbTrait;

/**
 * Functional tests for the menu-based breadcrumb builder.
 *
 * Copyright (C) 2022  Library Solutions, LLC (et al.).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * @coversDefaultClass \Drupal\crouton\MenuBasedBreadcrumbBuilder
 * @group crouton
 */
class MenuBasedBreadcrumbBuilderFunctionalTest extends BrowserTestBase {

  use AssertBreadcrumbTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'block',
    'crouton',
    'menu_link_content',
    'node',
  ];

  /**
   * Data provider for ::testBreadcrumbPrepend().
   */
  public function providerTestBreadcrumbPrepend() {
    return [
      'matching' => [TRUE],
      'not matching' => [FALSE],
    ];
  }

  /**
   * Test that menu-based breadcrumbs are displayed.
   */
  public function testBreadcrumbs() {
    $expected = [];

    $config_factory = \Drupal::configFactory();

    $crouton_settings = $config_factory->getEditable('crouton.settings');
    $crouton_settings->set('menu_name', 'main');
    $crouton_settings->save();

    $content_type = $this->drupalCreateContentType();

    for ($i = 1; $i <= 3; ++$i) {
      ($node = Node::create([
        'title' => "Test {$i}",
        'type' => $content_type->id(),
      ]))->save();

      ($link = MenuLinkContent::create([
        'title' => $node->label(),
        'parent' => isset($link) ? $link->getPluginId() : NULL,
        'provider' => 'menu_link_content',
        'menu_name' => 'main',
        'link' => [
          'uri' => "internal:/{$node->toUrl()->getInternalPath()}",
        ],
      ]))->save();

      $expected[$node->toUrl()->getInternalPath()] = $node->label();
    }

    array_pop($expected);

    $this->drupalPlaceBlock('system_breadcrumb_block');
    $this->assertBreadcrumb($node->toUrl(), $expected);
  }

  /**
   * Test the prepend of the front page to the breadcrumb trail.
   *
   * @dataProvider providerTestBreadcrumbPrepend
   *
   * @covers ::addMissingFrontLink
   */
  public function testBreadcrumbPrepend(bool $matching) {
    $expected = !$matching ? ['' => 'Home'] : [];

    $config_factory = \Drupal::configFactory();

    $crouton_settings = $config_factory->getEditable('crouton.settings');
    $crouton_settings->set('menu_name', 'main');
    $crouton_settings->set('prepend_front', TRUE);
    $crouton_settings->save();

    $content_type = $this->drupalCreateContentType();

    for ($i = 1; $i <= 3; ++$i) {
      ($node = Node::create([
        'title' => "Test {$i}",
        'type' => $content_type->id(),
      ]))->save();

      ($link = MenuLinkContent::create([
        'title' => $node->label(),
        'parent' => isset($link) ? $link->getPluginId() : NULL,
        'provider' => 'menu_link_content',
        'menu_name' => 'main',
        'link' => [
          'uri' => "internal:/{$node->toUrl()->getInternalPath()}",
        ],
      ]))->save();

      $expected[$node->toUrl()->getInternalPath()] = $node->label();
    }

    array_pop($expected);

    $system_site = $config_factory->getEditable('system.site');
    $system_site->set('page.front', $matching ? '/' . key($expected) : '/node');
    $system_site->save();

    $this->drupalPlaceBlock('system_breadcrumb_block');
    $this->assertBreadcrumb($node->toUrl(), $expected);
  }

}
